/*
 * nfsd.h	This program implements a user-space NFS server.
 *
 * Authors:	Mark A. Shand, May 1988
 *		Rick Sladkey, <jrs@world.std.com>
 *		Fred N. van Kempen, <waltje@uWalt.NL.Mugnet.ORG>
 *
 *		Copyright 1988 Mark A. Shand
 *		This software maybe be used for any purpose provided
 *		the above copyright notice is retained.  It is supplied
 *		as is, with no warranty expressed or implied.
 */

#include "system.h"

#include "mount.h"
#include "nfs_prot.h"
#ifdef NFSV3
#undef NFS_PROGRAM
#include "nfs3_prot.h"
#endif /* NFSV3 */

/* Make larger reads possible. Without crashing the machine :-) */
#undef NFS_MAXDATA
#define NFS_MAXDATA	(64 * 1024)

/* Maximum string size for gethostname() */
#define HOSTNAMESIZE 128

union argument_types {
	nfs_fh			nfsproc_getattr_2_arg;
	sattrargs		nfsproc_setattr_2_arg;
	diropargs		nfsproc_lookup_2_arg;
	nfs_fh			nfsproc_readlink_2_arg;
	readargs		nfsproc_read_2_arg;
	writeargs		nfsproc_write_2_arg;
	createargs		nfsproc_create_2_arg;
	diropargs		nfsproc_remove_2_arg;
	renameargs		nfsproc_rename_2_arg;
	linkargs		nfsproc_link_2_arg;
	symlinkargs		nfsproc_symlink_2_arg;
	createargs		nfsproc_mkdir_2_arg;
	diropargs		nfsproc_rmdir_2_arg;
	readdirargs		nfsproc_readdir_2_arg;
	nfs_fh			nfsproc_statfs_2_arg;
};

union result_types {
	attrstat		attrstat;
	diropres		diropres;
	readlinkres		readlinkres;
	readres			readres;
	nfsstat			nfsstat;
	readdirres		readdirres;
	statfsres		statfsres;
#ifdef NFSV3
        getattr3res             getattr3res;
        wccstat3                wccstat3;
        lookup3res              lookup3res;
        access3res              access3res;
        readlink3res            readlink3res;
        read3res                read3res;
        write3res               write3res;
        diropres3               diropres3;
        rename3res              rename3res;
        link3res                link3res;
        readdir3res             readdir3res;
        readdirplus3res         readdirplus3res;
        fsstat3res              fsstat3res;
        fsinfo3res              fsinfo3res;
        pathconf3res            pathconf3res;
        commit3res              commit3res;
#endif /* NFSV3 */
};

/* Global variables. */
extern union argument_types	argument;
extern union result_types	result;
extern int			need_reinit;
extern int			need_flush;
extern int			need_alarm;
extern time_t			nfs_dispatch_time;

/* Include the other module definitions. */
#include "auth.h"
#include "logging.h"

/* Global Function prototypes. */
extern void	nfs_dispatch(struct svc_req *, SVCXPRT *);
extern void	mallocfailed(void);
extern nfsstat	fh_getattr(nfs_fh *fh, fattr *attr,
					struct stat *stat_optimize,
					struct svc_req *rqstp);
extern nfsstat	fh_setattr(nfs_fh *fh, sattr *attr,
					struct stat *stat_optimize,
					struct svc_req *, int flags);
extern nfsstat	setattr(char *path, sattr *attr,
					struct stat *stat_optimize,
					struct svc_req *, int flags);
extern RETSIGTYPE reinitialize(int sig);
extern RETSIGTYPE flush_caches(int sig);
extern RETSIGTYPE setalarm(int sig);

#define SATTR_STAT		0x01
#define SATTR_CHOWN		0x02
#define SATTR_CHMOD		0x04
#define SATTR_SIZE		0x08
#define SATTR_UTIMES		0x10
#define SATTR_ALL		(~SATTR_STAT)

#ifndef HAVE_REALPATH
extern char *	realpath(const char *path, char *resolved_path);
#endif /* HAVE_REALPATH */

#ifndef HAVE_STRERROR
extern char *	strerror(int errnum);
#endif /* HAVE_STRERROR */

extern int nfsd_nfsproc_null_2(void *, struct svc_req *);
extern int nfsd_nfsproc_getattr_2(nfs_fh *, struct svc_req *);
extern int nfsd_nfsproc_setattr_2(sattrargs *, struct svc_req *);
extern int nfsd_nfsproc_root_2(void *, struct svc_req *);
extern int nfsd_nfsproc_lookup_2(diropargs *, struct svc_req *);
extern int nfsd_nfsproc_readlink_2(nfs_fh *, struct svc_req *);
extern int nfsd_nfsproc_read_2(readargs *, struct svc_req *);
extern int nfsd_nfsproc_writecache_2(void *, struct svc_req *);
extern int nfsd_nfsproc_write_2(writeargs *, struct svc_req *);
extern int nfsd_nfsproc_create_2(createargs *, struct svc_req *);
extern int nfsd_nfsproc_remove_2(diropargs *, struct svc_req *);
extern int nfsd_nfsproc_rename_2(renameargs *, struct svc_req *);
extern int nfsd_nfsproc_link_2(linkargs *, struct svc_req *);
extern int nfsd_nfsproc_symlink_2(symlinkargs *, struct svc_req *);
extern int nfsd_nfsproc_mkdir_2(createargs *, struct svc_req *);
extern int nfsd_nfsproc_rmdir_2(diropargs *, struct svc_req *);
extern int nfsd_nfsproc_readdir_2(readdirargs *, struct svc_req *);
extern int nfsd_nfsproc_statfs_2(nfs_fh *, struct svc_req *);

#ifdef NFSV3
extern int nfsd_nfsproc_null_3(void *, struct svc_req *);
extern int nfsd_nfsproc_getattr_3(nfs_fh3 *, struct svc_req *);
extern int nfsd_nfsproc_setattr_3(setattr3args *, struct svc_req *);
extern int nfsd_nfsproc_lookup_3(diropargs3 *, struct svc_req *);
extern int nfsd_nfsproc_access_3(access3args *, struct svc_req *);
extern int nfsd_nfsproc_readlink_3(nfs_fh3 *, struct svc_req *);
extern int nfsd_nfsproc_read_3(read3args *, struct svc_req *);
extern int nfsd_nfsproc_write_3(write3args *, struct svc_req *);
extern int nfsd_nfsproc_create_3(create3args *, struct svc_req *);
extern int nfsd_nfsproc_mkdir_3(mkdir3args *, struct svc_req *);
extern int nfsd_nfsproc_symlink_3(symlink3args *rgp, struct svc_req *);
extern int nfsd_nfsproc_mknod_3(mknod3args *, struct svc_req *);
extern int nfsd_nfsproc_remove_3(diropargs3 *, struct svc_req *);
extern int nfsd_nfsproc_rmdir_3(diropargs3 *, struct svc_req *);
extern int nfsd_nfsproc_rename_3(rename3args *, struct svc_req *);
extern int nfsd_nfsproc_link_3(link3args *, struct svc_req *);
extern int nfsd_nfsproc_readdir_3(readdir3args *, struct svc_req *);
extern int nfsd_nfsproc_readdirplus_3(readdirplus3args *, struct svc_req *);
extern int nfsd_nfsproc_fsstat_3(nfs_fh3 *, struct svc_req *);
extern int nfsd_nfsproc_fsinfo_3(nfs_fh3 *, struct svc_req *);
extern int nfsd_nfsproc_pathconf_3(nfs_fh3 *, struct svc_req *);
extern int nfsd_nfsproc_commit_3(commit3args *, struct svc_req *);
#endif /* NFSV3 */

#define   TRANSPORT_MASK  03
#define   NFS_OVER_TCP    02              /* use NFS over TCP/IP */
#define   NFS_OVER_UDP    01              /* use NFS over UDP/IP */

extern AUTH *create_authenticator_with_map(int authtype, int uid, int gid);

/* End of nfsd.h. */
